@@ -1,4 +1,4 @@
-# $Id: mkinfit.R 83 2013-06-18 14:08:11Z jranke $
+# $Id: mkinfit.R 118 2013-10-17 13:24:42Z jranke $
 
 # Copyright (C) 2010-2013 Johannes Ranke
 # Contact: jranke@uni-bremen.de
@@ -33,6 +33,8 @@ mkinfit <- function(mkinmod, observed,
   plot = FALSE, quiet = FALSE,
   err = NULL, weight = "none", scaleVar = FALSE,
   atol = 1e-8, rtol = 1e-10, n.outtimes = 100,
+  reweight.method = NULL,
+  reweight.tol = 1e-8, reweight.max.iter = 10,
   trace_parms = FALSE,
   ...)
 {
@@ -87,21 +89,26 @@ mkinfit <- function(mkinmod, observed,
   state.ini.fixed <- state.ini[fixed_initials]
   state.ini.optim <- state.ini[setdiff(names(state.ini), fixed_initials)]
 
-  # Preserve names of state variables before renaming initial state variable parameters
+  # Preserve names of state variables before renaming initial state variable
+  # parameters
   state.ini.optim.boxnames <- names(state.ini.optim)
+  state.ini.fixed.boxnames <- names(state.ini.fixed)
   if(length(state.ini.optim) > 0) {
-      names(state.ini.optim) <- paste(names(state.ini.optim), "0", sep="_")
+    names(state.ini.optim) <- paste(names(state.ini.optim), "0", sep="_")
+  }
+  if(length(state.ini.fixed) > 0) {
+    names(state.ini.fixed) <- paste(names(state.ini.fixed), "0", sep="_")
   }
 
   # Decide if the solution of the model can be based on a simple analytical
   # formula, the spectral decomposition of the matrix (fundamental system)
   # or a numeric ode solver from the deSolve package
   if (!solution_type %in% c("auto", "analytical", "eigen", "deSolve"))
-      stop("solution_type must be auto, analytical, eigen or de Solve")
+     stop("solution_type must be auto, analytical, eigen or de Solve")
   if (solution_type == "analytical" && length(mkinmod$map) > 1)
-      stop("Analytical solution not implemented for models with metabolites.")
+     stop("Analytical solution not implemented for models with metabolites.")
   if (solution_type == "eigen" && !is.matrix(mkinmod$coefmat))
-      stop("Eigenvalue based solution not possible, coefficient matrix not present.")
+     stop("Eigenvalue based solution not possible, coefficient matrix not present.")
   if (solution_type == "auto") {
     if (length(mkinmod$map) == 1) {
       solution_type = "analytical"
@@ -109,10 +116,10 @@ mkinfit <- function(mkinmod, observed,
       if (is.matrix(mkinmod$coefmat)) {
 	solution_type = "eigen"
         if (max(observed$value, na.rm = TRUE) < 0.1) {
-	  stop("The combination of small observed values (all < 0.1) and solution_type = eigen is error-prone")
-	}
+          stop("The combination of small observed values (all < 0.1) and solution_type = eigen is error-prone")
+        }
       } else {
-	solution_type = "deSolve"
+        solution_type = "deSolve"
       }
     }
   }
@@ -125,25 +132,28 @@ mkinfit <- function(mkinmod, observed,
   {
     assign("calls", calls+1, inherits=TRUE) # Increase the model solution counter
 
-    # Trace parameter values if quiet is off
+    # Trace parameter values if requested
     if(trace_parms) cat(P, "\n")
 
     # Time points at which observed data are available
     # Make sure we include time 0, so initial values for state variables are for time 0
-    outtimes = sort(unique(c(observed$time,
-		      seq(min(observed$time), max(observed$time), length.out=n.outtimes))))
+    outtimes = sort(unique(c(observed$time, seq(min(observed$time),
+                                                max(observed$time),
+                                                length.out = n.outtimes))))
 
     if(length(state.ini.optim) > 0) {
       odeini <- c(P[1:length(state.ini.optim)], state.ini.fixed)
-      names(odeini) <- c(state.ini.optim.boxnames, names(state.ini.fixed))
-    } else odeini <- state.ini.fixed
+      names(odeini) <- c(state.ini.optim.boxnames, state.ini.fixed.boxnames)
+    } else odeini <- state.ini.fixed.boxnames
 
     odeparms <- c(P[(length(state.ini.optim) + 1):length(P)], parms.fixed)
 
     parms <- backtransform_odeparms(odeparms, mod_vars)
 
     # Solve the system with current transformed parameter values
-    out <- mkinpredict(mkinmod, parms, odeini, outtimes, solution_type = solution_type, atol = atol, rtol = rtol, ...)
+    out <- mkinpredict(mkinmod, parms, odeini, outtimes, 
+                       solution_type = solution_type, 
+                       atol = atol, rtol = rtol, ...)
 
     assign("out_predicted", out, inherits=TRUE)
 
@@ -169,7 +179,7 @@ mkinfit <- function(mkinmod, observed,
         names(col_obs) <- names(pch_obs) <- names(lty_obs) <- obs_vars
         for (obs_var in obs_vars) {
           points(subset(observed, name == obs_var, c(time, value)), 
-            pch = pch_obs[obs_var], col = col_obs[obs_var])
+                 pch = pch_obs[obs_var], col = col_obs[obs_var])
         }
         matlines(out_plot$time, out_plot[-1], col = col_obs, lty = lty_obs)
         legend("topright", inset=c(0.05, 0.05), legend=obs_vars, 
@@ -180,7 +190,43 @@ mkinfit <- function(mkinmod, observed,
     }
     return(mC)
   }
-  fit <- modFit(cost, c(state.ini.optim, parms.optim), method = method.modFit, control = control.modFit, ...)
+
+  fit <- modFit(cost, c(state.ini.optim, parms.optim), 
+                method = method.modFit, control = control.modFit, ...)
+
+  # Reiterate the fit until convergence of the variance components (IRLS)
+  # if requested by the user
+  weight.ini = weight
+  if (!is.null(err)) weight.ini = "manual"
+
+  if (!is.null(reweight.method)) {
+    if (reweight.method != "obs") stop("Only reweighting method 'obs' is implemented")
+    if(!quiet) {
+      cat("IRLS based on variance estimates for each observed variable\n")
+    }
+    if (!quiet) {
+      cat("Initial variance estimates are:\n")
+      print(signif(fit$var_ms_unweighted, 8))
+    }
+    reweight.diff = 1
+    n.iter <- 0
+    if (!is.null(err)) observed$err.ini <- observed[[err]]
+    err = "err.irls"
+    while (reweight.diff > reweight.tol & n.iter < reweight.max.iter) {
+      n.iter <- n.iter + 1
+      sigma.old <- sqrt(fit$var_ms_unweighted)
+      observed[err] <- sqrt(fit$var_ms_unweighted)[as.character(observed$name)]
+      fit <- modFit(cost, fit$par, method = method.modFit,
+                    control = control.modFit, ...)
+      reweight.diff = sum((sqrt(fit$var_ms_unweighted) - sigma.old)^2)
+      if (!quiet) {
+        cat("Iteration", n.iter, "yields variance estimates:\n")
+        print(signif(fit$var_ms_unweighted, 8))
+        cat("Sum of squared differences to last variance estimates:",
+            signif(reweight.diff, 2), "\n")
+      }
+    }
+  }
 
   # We need to return some more data for summary and plotting
   fit$solution_type <- solution_type
@@ -194,27 +240,39 @@ mkinfit <- function(mkinmod, observed,
   fit$predicted <- mkin_wide_to_long(out_predicted, time = "time")
 
   # Collect initial parameter values in two dataframes
-  fit$start <- data.frame(initial = c(state.ini.optim, 
+  fit$start <- data.frame(value = c(state.ini.optim, 
 		  backtransform_odeparms(parms.optim, mod_vars)))
-  fit$start$type = c(rep("state", length(state.ini.optim)), rep("deparm", length(parms.optim)))
+  fit$start$type = c(rep("state", length(state.ini.optim)), 
+                     rep("deparm", length(parms.optim)))
   fit$start$transformed = c(state.ini.optim, parms.optim)
 
   fit$fixed <- data.frame(
     value = c(state.ini.fixed, parms.fixed))
-  fit$fixed$type = c(rep("state", length(state.ini.fixed)), rep("deparm", length(parms.fixed)))
+  fit$fixed$type = c(rep("state", length(state.ini.fixed)), 
+                     rep("deparm", length(parms.fixed)))
 
   bparms.optim = backtransform_odeparms(fit$par, mod_vars)
-  bparms.fixed = backtransform_odeparms(c(state.ini.fixed, parms.fixed), mod_vars)
+  bparms.fixed = backtransform_odeparms(c(state.ini.fixed, parms.fixed), 
+                                        mod_vars)
   bparms.all = c(bparms.optim, bparms.fixed)
 
-  # Collect observed, predicted and residuals
+  # Collect observed, predicted, residuals and weighting
   data <- merge(fit$observed, fit$predicted, by = c("time", "name"))
-  names(data) <- c("time", "variable", "observed", "predicted")
-  data$residual <- data$observed - data$predicted
-  data$variable <- ordered(data$variable, levels = obs_vars)
-  fit$data <- data[order(data$variable, data$time), ]
+  data$name <- ordered(data$name, levels = obs_vars)
+  data <- data[order(data$name, data$time), ]
+
+  fit$data <- data.frame(time = data$time,
+                         variable = data$name,
+                         observed = data$value.x,
+                         predicted = data$value.y)
+  fit$data$residual <- fit$data$observed - fit$data$predicted
+  if (!is.null(data$err.ini)) fit$data$err.ini <- data$err.ini
+  if (!is.null(err)) fit$data[[err]] <- data[[err]]
+
   fit$atol <- atol
   fit$rtol <- rtol
+  fit$weight.ini <- weight.ini
+  fit$reweight.method <- reweight.method
 
   # Return all backtransformed parameters for summary
   fit$bparms.optim <- bparms.optim 
@@ -270,6 +328,8 @@ summary.mkinfit <- function(object, data = TRUE, distimes = TRUE, alpha = 0.05,
 	  date.summary = date(),
 	  solution_type = object$solution_type,
 	  use_of_ff = object$mkinmod$use_of_ff,
+    weight.ini = object$weight.ini,
+    reweight.method = object$reweight.method,
     residuals = object$residuals,
     residualVariance = resvar,
     sigma = sqrt(resvar),
@@ -316,6 +376,11 @@ print.summary.mkinfit <- function(x, digits = max(3, getOption("digits") - 3), .
   cat("\nMethod used for solution of differential equation system:\n")
   cat(x$solution_type, "\n")
 
+  cat("\nWeighting:", x$weight.ini)
+  if(!is.null(x$reweight.method)) cat(" then iterative reweighting method",
+                                      x$reweight.method)
+  cat("\n")
+
   cat("\nStarting values for optimised parameters:\n")
   print(x$start)
 
@@ -371,3 +436,4 @@ print.summary.mkinfit <- function(x, digits = max(3, getOption("digits") - 3), .
 
   invisible(x)
 }
+# vim: set ts=2 sw=2 expandtab:
